/* misccmds:
 *	More monitor commands...
 *
 *	General notice:
 *	This code is part of a boot-monitor package developed as a generic base
 *	platform for embedded system designs.  As such, it is likely to be
 *	distributed to various projects beyond the control of the original
 *	author.  Please notify the author of any enhancements made or bugs found
 *	so that all may benefit from the changes.  In addition, notification back
 *	to the author will allow the new user to pick up changes that may have
 *	been made by other users after this version of the code was distributed.
 *
 *	Note1: the majority of this code was edited with 4-space tabs.
 *	Note2: as more and more contributions are accepted, the term "author"
 *		   is becoming a mis-representation of credit.
 *
 *	Original author:	Ed Sutter
 *	Email:				ed.sutter@alcatel-lucent.com
 *	Phone:				908-582-2351
 */
#include "config.h"
#include "stddefs.h"
#include "tfs.h"
#include "tfsprivate.h"
#include "genlib.h"
#include "ether.h"
#include "devices.h"
#include "cli.h"
#include "ctype.h"
#include "cpuio.h"

int		(*extgetUsrLvl)(void);

int
getUsrLvl(void)
{
	return(MAXUSRLEVEL);
}

#if INCLUDE_VERSIONCMD
char *VersionHelp[] = {
		"Version information",
		"",
		0,
};

int
Version(int argc,char *argv[])
{
	extern	void ShowVersion(void);

	ShowVersion();
	return(CMD_SUCCESS);
}
#endif

#if INCLUDE_TFSSCRIPT
char *EchoHelp[] = {
	"Print string to local terminal",
	"[arg1] ... [argn]",
	0,
};

int
Echo(int argc,char *argv[])
{
	char	*cp;
	int		i, done;

	for(i=optind;i<argc;i++) {
		cp = argv[i];
		done = 0;
		while(!done && *cp)
			putchar(*cp++);
		if (i != argc-1) {
			putchar(' ');
		}
	}
	putchar('\n');
	flush_console_out();
	return(CMD_SUCCESS);
}
#endif

#if INCLUDE_CALLCMD

/* vcall():
 * Simulate a reset through the vector table.
 * This function loads both the SP and PC (similar
 * to what would be done at reset).
 * If VCALL_RAMINIT is defined, then this function will 
 * also clear the RAM between _vcallramstart & _vcallramend.
 * These symbols should be set up in the linker file
 * to be the full range of system ram.
 */
__attribute__((naked)) long vcall(uint32_t stackptr, uint32_t prgmctr)
{
#if INCLUDE_VCALLRAMINIT
	// Zero fill all ram, using vcallramstart and vcallramend...
  __asm volatile ("  ldr  r2, =_vcallramstart");
  __asm volatile ("  ldr  r3, =_vcallramend");
  __asm volatile ("  mov  r4, #0");
  __asm volatile ("  .thumb_func");
  __asm volatile ("zero_loop:");
  __asm volatile ("  cmp     r2, r3");
  __asm volatile ("  it      lt");
  __asm volatile ("  strlt   r4, [r2], #4");
  __asm volatile ("  blt     zero_loop");
#endif

	// Load up new sp from 'stackptr' parameter...
  __asm volatile("  msr msp, r0");	// stackptr = r0
  __asm volatile("  msr psp, r0");

	// Transfer 'prgmctr' parameter to link register...
  __asm volatile("  mov lr,  r1");	// prgmctr = r1

	// Clear GPRs... 
  __asm volatile ("  mov r0,#0");
  __asm volatile ("  mov r1,#0");
  __asm volatile ("  mov r2,#0");
  __asm volatile ("  mov r3,#0");
  __asm volatile ("  mov r4,#0");
  __asm volatile ("  mov r5,#0");
  __asm volatile ("  mov r6,#0");
  __asm volatile ("  mov r7,#0");
  __asm volatile ("  mov r8,#0");
  __asm volatile ("  mov r9,#0");
  __asm volatile ("  mov r10,#0");
  __asm volatile ("  mov r11,#0");
  __asm volatile ("  mov r12,#0");

  // With lr now loaded with new program counter, 
  // jump there...
  __asm volatile("  bx lr");
  __asm volatile("  nop");
	return(0);
}

/* Call():
 *	This function is called when the user wants to execute an 
 *	embedded function. 
 *  The the argument is preceded by an ampersand, then a pointer
 *  to the argument is passed to the function instead of a 
 *	strtol() conversion.
 */
char *CallHelp[] = {
	"Call embedded function",
	"{addr} [args]",
	0,
};

int
Call(int argc,char *argv[])
{
	int		i, j;
	long	args[10], func_addr;
	int		(*func)(long,long,long,long);

#if INCLUDE_MONLIB
	int opt, monargs = 0;

	while((opt=getopt(argc,argv,"A")) != -1) {
		switch(opt) {
		case 'A':
			monargs = 1;
			break;
		default:
			return(CMD_PARAM_ERROR);
		}
	}
#endif

	if ((argc < optind+1) || (argc > optind+11))
		return(CMD_PARAM_ERROR);

	/* If first char is '*', then dereference the pointer;
	 * If first char is 'v', then assume the address points to a vector table;
	 * otherwise, just use the address as the address.
	 */
	if (argv[optind][0] == '*') {
	    func_addr = strtol(&argv[optind][1],(char **)0,0);
		func_addr = *(long *)func_addr;
	}
	else if (argv[optind][0] == 'v') {
		ulong *vtable = (ulong *)strtol(&argv[optind][1],(char **)0,0);
		ulong sp = *vtable++;
		func_addr = *vtable;

		uninitCPUio();
		vcall(sp,func_addr);
		return(CMD_SUCCESS);	// Should not get here
	}
	else
	    func_addr = strtol(argv[optind],(char **)0,0);

#ifdef ADJUST_THUMB_ADDRESS // If THUMB, then add 1...
    func_addr = ADJUST_THUMB_ADDRESS(func_addr);
#endif

    func = (int(*)(long,long,long,long))func_addr;

	if ((func == 0) && (isdigit(argv[optind][0]) == 0)) {
		return(CMD_PARAM_ERROR);
	}

#if INCLUDE_MONLIB
	/* If -A option is specified, then we push all command line args
	 * through putargv() for use by getargv() in the application;
	 * otherwise just retrieve and convert args from command line.
	 */
	if (monargs) {
		for(j=0,i=optind;i<argc;i++,j++)
			putargv(j,argv[i]);
		putargv(j,(char *)0);
	}
	else
#endif
	{
		/* If the first character of the argument is an ampersand (&),
		 * then a pointer to the argument is passed; otherwise, the
		 * argument is converted to a long integer using strtol()...
		 */
		for(j=0,i=optind+1;i<argc;i++,j++) {
			if (argv[i][0] == '&')
				args[j] = (ulong)&argv[i][1];
			else
				args[j] = strtol(argv[i],(char **)0,0);
		}
	}

	func(args[0],args[1],args[2],args[3]);

	return(CMD_SUCCESS);
}
#endif

#if INCLUDE_RESETCMD
/* Reset():
 *	Used to re-initialize the monitor through the command interface.
 */

char *ResetHelp[] = {
	"Reset monitor firmware",
	"-[x]",
#if INCLUDE_VERBOSEHELP
	" -x      app_exit",
#endif
	0,
};

int
Reset(int argc,char *argv[])
{
	extern	void appexit(int);
	int	opt;

	intsoff();

	/* For some systems, the reset occurs while characters are in the
	 * UART FIFO (so they don't get printed).  Adding this delay will
	 * hopefully allow the characters in the FIFO to drain...
	 */
	monDelay(250);

	while((opt=getopt(argc,argv,"x")) != -1) {
		switch(opt) {
		case 'x':
			appexit(0);
			break;
		default:	
			return(CMD_PARAM_ERROR);
		}
	}
	target_reset();
	return(CMD_SUCCESS);
}
#endif

#if INCLUDE_WHATCMD

#define WHATSEARCH	1
#define WHATMATCH	2
#define WHATPREFIX	"@(#)"

int
whatsearch(char *bp, char *end)
{
	int		state, idx, tot;
	char	whatbuf[64], *whatstring, *wsp;

	idx = tot = 0;
	state = WHATSEARCH;
	whatstring = wsp = WHATPREFIX;
	while(bp < end) {
		switch(state) {
		case WHATSEARCH:
			if (*bp == *wsp) {
				wsp++;
				if (*wsp == 0) {
					wsp = whatstring;
					state = WHATMATCH;
					idx = 0;
					printf("  ");
				}
			}
			else
				wsp = whatstring;
			break;
		case WHATMATCH:
			if (isprint(*bp)) {
				printf("%c",*bp);
				if (idx < sizeof(whatbuf)-1)
					whatbuf[idx++] = *bp;
			}
			else {
				printf("\n");
				if (idx < sizeof(whatbuf))
					whatbuf[idx] = 0;
				state = WHATSEARCH;
				tot++;
			}
			break;
		}
		bp++;
	}
	return(tot);
}

char *WhatHelp[] = {
	"Find 'what' strings",
	"{begin} {end}",
	0,
};

int
WhatCmd(int argc,char **argv)
{
	char	*end, *begin;

	if (argc != 3)
		return(CMD_PARAM_ERROR);

	begin = (char *)strtol(argv[1],0,0);
	end = (char *)strtol(argv[2],0,0);

	whatsearch(begin,end);
	return(CMD_SUCCESS);
}
#endif
